{
 description = "docman is a document management tool";

  inputs = {
    nixpkgs.url = "nixpkgs/nixos-23.05";    
  };

  outputs = { self, nixpkgs,  ... }:
    let

      lastModifiedDate = self.lastModifiedDate or self.lastModified or "19700101";
      supportedSystems = [ "x86_64-linux" "x86_64-darwin" "aarch64-linux" "aarch64-darwin" ];

      forAllSystems = nixpkgs.lib.genAttrs supportedSystems;

      nixpkgsFor = forAllSystems (system: import nixpkgs { inherit system; });
      pkgs = import nixpkgs { system = "x86_64-linux"; };
      
    in {

      packages = forAllSystems (system:
        let
           pkgs = nixpkgsFor.${system};
  
        in {
          docman = pkgs.buildGoModule {
          
            name = "docman";
            version = "${self.ref}";
            
            src = ././source;

            vendorSha256 = "sha256-81IBG8H1jsYKD+tmFeCpAWFdz6cuQDt9r+FV56xboBU=";
            
            ldflags = [
                 "-X 'gitlab.schukai.com/oss/utilities/documentation-manager/release.version=${self.ref}' -X 'gitlab.schukai.com/oss/utilities/documentation-manager/release.build=${self.lastModified}'"
                      ];
             

            meta = with nixpkgs.legacyPackages.${system}.lib; {
                description = "docman is a document management tool";
                homepage = "https://gitlab.schukai.com/oss/utilities/documentation-manager";
                license = licenses.mit;
                maintainers = with maintainers; [ "schukai GmbH" ];
              };    

          };
        });
      
      devShells = forAllSystems (system:
        let 
          pkgs = nixpkgsFor.${system};
        in
        {
          default = pkgs.mkShell {
            buildInputs = with pkgs; [ go gopls gotools go-tools ];
          };
        });


      defaultPackage = forAllSystems (system: self.packages.${system}.docman);
    };
}
