// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package document

import (
	"gitlab.schukai.com/oss/utilities/documentation-manager/utils"
	"path"
	"path/filepath"
	"regexp"
	"strings"
)

type foundedTitleInfoStruct struct {
	level int
	match string
	title string
}

type foundedTitleInfo []foundedTitleInfoStruct

func convertHeadlines(content string, level, startLevel int) string {
	d := 0
	info, smallestLevelInTheDoc := calculateLevel(content)
	if startLevel == 0 {
		d = level - smallestLevelInTheDoc
	} else {
		d = startLevel - smallestLevelInTheDoc
	}

	if d < 0 {
		d = 0
	}

	for _, i := range info {
		fill := strings.Repeat("#", d+i.level)
		nw := fill + " " + i.title + "\n"
		content = strings.Replace(content, i.match, nw, 1)
	}

	return content
}

func calculateLevel(content string) (foundedTitleInfo, int) {
	regEx := regexp.MustCompile(`(?m)^(?P<match>(?P<level>#+)\s+(?P<title>[^\n]*))`)
	matches := regEx.FindAllStringSubmatch(content, -1)
	info := foundedTitleInfo{}
	if matches == nil {
		return info, -1
	}

	smallestLevelInTheDoc := 999

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range regEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		info = append(info, foundedTitleInfoStruct{
			level: len(result["level"]),
			match: result["match"],
			title: result["title"],
		})

		if len(result["level"]) < smallestLevelInTheDoc {
			smallestLevelInTheDoc = len(result["level"])
		}

	}

	return info, smallestLevelInTheDoc
}

func convertImages(content string, absolute string) string {

	regEx := regexp.MustCompile(`(?P<match>\!\[(?P<label>[^]]*)\]\((?P<path>[^)]*)\))`)

	matches := regEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range regEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if utils.IsUrl(result["path"]) {
			continue
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		p := filepath.Join(absolute, result["path"])
		path := path.Clean(p)
		content = strings.Replace(content, result["match"], "!["+result["label"]+"]("+path+")", -1)

	}

	return content
}
